const path = require('path')
const utils = require('./utils')
const { CompareVars } = require('./constants')
const VSelector = require('./VSelector')
const expects = require('./bom/expects')
const waitFors = require('./bom/waitFors')
const location = require('./bom/location')
const mock = require('./mock')

browser = null
page = null

utils.screenshotSaveFolder = path.join(process.cwd(), 'screenshot')
utils.mockDataFolder = path.join(process.cwd(), 'mock')

function TestKit(selector) {
    if (!selector) {
        throw new Error('[TestKit] selector is required')
    }
    return new VSelector(selector)
}

TestKit.constants = CompareVars
TestKit.browser = null
TestKit.setBrowser = async (b) => {
    browser = TestKit.browser = b

    await TestKit.setCurrentPage((await browser.pages())[0])

    initPage()

    browser.on('targetcreated', (target) => {
        ;(async () => {
            await initPage(await target.page())
        })()
    })
}

TestKit.page = null
TestKit.setCurrentPage = async (p) => {
    page = TestKit.page = p
}

TestKit.setDebugMode = (yes) => {
    utils.debugMode = yes
}

TestKit.setScreenshotFolder = (folderPath) => {
    utils.screenshotSaveFolder = folderPath
}

TestKit.setMockDataFolder = (folderPath) => {
    utils.mockDataFolder = folderPath
}

TestKit.setMockHeaders = (headers) => {
    utils.mockHeaders = headers
}

TestKit.mock = (maps, headers) => {
    const spinner = utils.log(`mocking for request`, { mockMaps: maps })
    return mock(maps, headers)
        .then(() => {
            spinner.succeed()
        })
        .catch((e) => {
            spinner.fail(e)
        })
}

function initPage(p) {
    // TODO: 可能会比较慢，导致 $Z undifined，需要一个机制去告知用例执行时机
    ;(p ? p : page).on('console', (msg) => {
        const jsh = msg.args()
        for (let x of jsh) {
            const text = x.toString().replace(/JSHandle\:/g, '')
            if (text.indexOf('[test]') === 0) {
                console.log(` ${text.replace(/\[test\]/g, '')} `.bgYellow)
            }
        }
    })
    ;(p ? p : page).on('load', async (response) => {
        await page.addScriptTag({
            path: path.join(__dirname, '../browser/$Z.js')
        })
    })
}

TestKit.reload = async (opts) => {
    const spinner = utils.log(`reload page`)
    try {
        const res = await TestKit.page.reload(opts)
        await page.waitForFunction('window.$Z')
        spinner.succeed()
        return res
    } catch (e) {
        spinner.fail(e)
    }
}

TestKit.title = async () => {
    const spinner = utils.log(`get page title`)
    const res = await page.title()
    if (res) {
        spinner.succeed(res)
    } else {
        spinner.fail()
    }
}

TestKit.findTarget = utils.findTarget
TestKit.closeTarget = utils.closeTarget
TestKit.location = location

TestKit.delay = (ms) => {
    const spinner = utils.log(`delay ${ms || 1000}ms`)
    return new Promise((r, rj) => {
        setTimeout(() => {
            r()
        }, ms || 1000)
    })
        .then(() => {
            spinner.succeed()
        })
        .catch((e) => {
            spinner.fail(e)
        })
}

utils.defineFreezedProps(TestKit, { waitFor: waitFors, expect: expects })

Object.freeze(TestKit.waitFor)

module.exports = TestKit
